------------------------------------------------------------
-- Container.lua
--
-- Abin
-- 2012/1/03
------------------------------------------------------------

local _G = _G
local RegisterStateDriver = RegisterStateDriver
local UnregisterStateDriver = UnregisterStateDriver
local GetNumRaidMembers = GetNumRaidMembers
local GetNumPartyMembers = GetNumPartyMembers

local _, addon = ...

local frame = CreateFrame("Frame", "CompactRaidMainFrame", UIParent, "SecureFrameTemplate")
frame:SetSize(16, 16)
frame:SetPoint("CENTER")
frame:SetFrameStrata("BACKGROUND")
frame:SetMovable(true)
frame:SetUserPlaced(true)
frame:SetClampedToScreen(true)

function addon:GetMainFrame()
	return frame
end

local function DragFrame_OnDragStart(self)
	if not addon.db.lock then
		frame:StartMoving()
	end
end

local function DragFrame_OnDragStop(self)
	frame:StopMovingOrSizing()
end

function addon:RegisterMainFrameMover(frame)
	frame:EnableMouse(true)
	frame:RegisterForDrag("LeftButton")
	frame:SetScript("OnDragStart", DragFrame_OnDragStart)
	frame:SetScript("OnDragStop", DragFrame_OnDragStop)
end

function addon:UpdateScale()
	frame:SetScale(addon.db.scale / 100)
end

function addon:UpdateVisibility()
	local single = not addon.db.hidesolo
	local party = not addon.db.hideparty

	if single and party then
		UnregisterStateDriver(frame, "visibility")
		frame:Show()
	elseif party then
		RegisterStateDriver(frame, "visibility", "[group] show; hide")
	elseif single then
		RegisterStateDriver(frame, "visibility", "[group:raid] show; [nogroup] show; hide")
	else
		RegisterStateDriver(frame, "visibility", "[group:raid] show; hide")
	end
end

function addon:UpdateSpacing()
	addon:UpdateSoloSpacing()
	addon:UpdatePartySpacing()
	addon:UpdateRaidGroupSpacing()
end

local container = CreateFrame("Frame", frame:GetName().."Container", frame)

function addon:GetContainerFrame()
	return container
end

container:SetFrameStrata("BACKGROUND")
container:SetSize(16, 16)
container:SetPoint("TOPLEFT")

local backdrop = { bgFile = "Interface\\DialogFrame\\UI-DialogBox-Background", edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16, insets = {left = 5, right = 5, top = 5, bottom = 5 }}
local containerBorder = CreateFrame("Frame", container:GetName().."Border", container)
containerBorder:SetBackdrop(backdrop)
containerBorder:SetPoint("TOPLEFT", -12, 12)
containerBorder:SetPoint("BOTTOMRIGHT", 13, -14)
containerBorder:SetFrameStrata("BACKGROUND")
containerBorder:SetFrameLevel(1)
addon:RegisterMainFrameMover(containerBorder)

function addon:SetTooltipAnchor()
	local right = containerBorder:GetLeft() < 300
	local bottom = GetScreenHeight() - containerBorder:GetTop() < 360

	local pointX = right and "LEFT" or "RIGHT"
	local pointY = bottom and "TOP" or "BOTTOM"
	local relPointX = right and "RIGHT" or "LEFT"
	local relPointY = bottom and "BOTTOM" or "TOP"

	GameTooltip:SetOwner(containerBorder, "ANCHOR_NONE")
	GameTooltip:ClearAllPoints()
	GameTooltip:SetPoint(pointY..pointX, containerBorder, relPointY..relPointX, right and 4 or -4, bottom and -4 or 4)
end

local needCalcSize = 1
function addon:UpdateContainerSize()
	needCalcSize = 1
end

function addon:UpdateContainerAlpha()
	container:SetAlpha(addon.db.containerAlpha / 100)
end

function addon:UpdateContainerBorderSize()
	local size = addon.db.containerBorderSize
	containerBorder:ClearAllPoints()
	containerBorder:SetPoint("TOPLEFT", -size, size)
	containerBorder:SetPoint("BOTTOMRIGHT", size + 1, -size - 1)
	addon:UpdateToolFramePosition()
end

function addon:SetContainerArtwork(background, border)
	backdrop.bgFile = background
	backdrop.edgeFile = border
	containerBorder:SetBackdrop(backdrop)
end

function addon:ImmediatelyUpdateContainerSize()
	needCalcSize = nil
	local x, y = addon.db.width, addon.db.height
	local width, height
	if GetNumRaidMembers() > 0 then
		width, height = addon:GetRaidGroupVisibleSize()
	elseif GetNumPartyMembers() > 0 then
		width, height = addon:GetPartyFrameVisibleSize()
	else
		width, height = addon:GetSoloFrameVisibleSize()
	end

	if width < x then
		width = x
	end

	if height < y then
		height = y
	end

	container:SetSize(width, height)
	frame:SetClampRectInsets(0, width - 16, 0, 16 - height) -- Is this function protected? Currently not, but it should be...
end

local updateElapsed = 0
container:SetScript("OnUpdate", function(self, elapsed)
	updateElapsed = updateElapsed + elapsed
	if updateElapsed < 0.2 then
		return
	end
	updateElapsed = 0
	if needCalcSize and addon:IsInitialized() then
		addon:ImmediatelyUpdateContainerSize()
	end
end)