------------------------------------------------------------
-- Core.lua
--
-- Abin
-- 2012/1/03
------------------------------------------------------------

local _G = _G
local DEFAULT_CHAT_FRAME = DEFAULT_CHAT_FRAME
local tostring = tostring
local tonumber = tonumber
local type = type
local format = format
local pcall = pcall
local wipe = wipe
local pairs = pairs
local InCombatLockdown = InCombatLockdown
local select = select
local tinsert = tinsert
local ipairs = ipairs
local tremove = tremove

local addonName, addon = ...
_G["CompactRaid"] = addon
addon.version = GetAddOnMetadata(addonName, "Version") or "2.0"
addon.numericVersion = tonumber(addon.version) or 2.0
addon.name = "CompactRaid"
addon.db = {}
addon.chardb = {}

local L = addon.L

function addon:Print(msg, r, g, b)
	DEFAULT_CHAT_FRAME:AddMessage("|cffffff78CompactRaid:|r "..tostring(msg), r or 0.5, g or 0.75, b or 1)
end

function addon:CheckVersion(version, errorTitle)
	if type(version) == "string" then
		version = tonumber(version)
	end

	if type(version) ~= "number" or version <= addon.numericVersion then
		return 1 -- Passed
	end

	if errorTitle then
		addon:Print(format(L["version error"], tostring(errorTitle), tostring(version), addon.version), 1, 0, 0)
	end
end

function addon:pcall(func, ...)
	if self.db.debugmode then
		-- debug mode: captures errors normally
		func(...)
		return 1
	else
		-- release mode: prints errors on ChatFrame1 and let go
		local valid, err = pcall(func, ...)
		if not valid then
			self:Print(err, 1, 0, 0)
		end
		return valid, err
	end
end

function addon:CloneTable(src, dest)
	if src == dest or type(src) ~= "table" then
		return
	end

	if type(dest) ~= "table" then
		dest = {}
	end

	wipe(dest)
	local k, v
	for k, v in pairs(src) do
		if type(v) == "table" then
			dest[k] = addon:CloneTable(v)
		else
			dest[k] = v
		end
	end
	return dest
end

function addon:NormalizeNumber(number, low, high, default)
	if type(number) ~= "number" then
		return default
	end
	return number >= low and number <= high and number or default
end

------------------------------------------------------------
-- For combat-safe calls, new feature added in 1.3
------------------------------------------------------------

local pendingCalls = {}

local function InvokeGlobalCall(func, ...)
	if type(func) == "string" then
		addon:pcall(_G[func], ...)
	else
		addon:pcall(func, ...)
	end
end

local function InvokeMethodCall(object, method, ...)
	addon:pcall(object[method], object, ...)
end

-- Issue a combat-safe call, it calls directly if not in combat and returns nothing,
-- or pending the call and returns the pending handle, the call will be invoked when
-- combat ends. The returned handle is only used to cancel the pending call.

-- handle = addon:CombatSafeCall("func", ...) -- Using a global function name
-- handle = addon:CombatSafeCall(func, ...) -- Using a function
-- handle = addon:CombatSafeCall(object, "method", ...) -- Using an object and its member method name

function addon:CombatSafeCall(object, method, ...)
	if InCombatLockdown() then
		local data = { object = object }
		data.isMethod = type(object) == "table"
		local count
		if data.isMethod then
			data.method = method
			count = select("#", ...)
		else
			count = select("#", method, ...)
		end

		if count > 0 then
			data.hasArgs = count
			local i
			for i = 1, count do
				if data.isMethod then
					data["arg"..i] = select(i, ...)
				else
					data["arg"..i] = select(i, method, ...)
				end
			end
		end

		tinsert(pendingCalls, data)
		return data
	else
		if type(object) == "table" then
			InvokeMethodCall(object, method, ...)
		else
			InvokeGlobalCall(object, method, ...)
		end
	end
end

-- Cancels a pending call
function addon:CancelPendingCombatSafeCall(handle)
	if InCombatLockdown() and type(data) == "table" then
		local id, data
		for id, data in ipairs(pendingCalls) do
			if data == handle then
				tremove(pendingCalls, id)
				return 1
			end
		end
	end
end

local frame = CreateFrame("Frame")
frame:RegisterEvent("PLAYER_REGEN_ENABLED")
frame:SetScript("OnEvent", function()
	local i, data
	for i = 1, #pendingCalls do -- It is important to keep the calling orders!
		local data = pendingCalls[i]
		if data.isMethod then
			if data.hasArgs then
				InvokeMethodCall(data.object, data.method, data.arg1, data.arg2, data.arg3, data.arg4, data.arg5, data.arg6, data.arg7, data.arg8, data.arg9, data.arg10, data.arg11, data.arg12, data.arg13, data.arg14, data.arg15, data.arg16, data.arg17, data.arg18, data.arg19, data.arg20)
			else
				InvokeMethodCall(data.object, data.method)
			end
		else
			if data.hasArgs then
				InvokeGlobalCall(data.object, data.arg1, data.arg2, data.arg3, data.arg4, data.arg5, data.arg6, data.arg7, data.arg8, data.arg9, data.arg10, data.arg11, data.arg12, data.arg13, data.arg14, data.arg15, data.arg16, data.arg17, data.arg18, data.arg19, data.arg20)
			else
				InvokeGlobalCall(data.object)
			end
		end
	end
	wipe(pendingCalls)
end)