------------------------------------------------------------
-- OptionTemplates.lua
--
-- Abin
-- 2010/10/31
------------------------------------------------------------

local _G = _G
local type = type
local tostring = tostring
local tonumber = tonumber
local pairs = pairs
local CreateFrame = CreateFrame
local StaticPopup_Hide = StaticPopup_Hide
local GameTooltip = GameTooltip
local NORMAL_FONT_COLOR = NORMAL_FONT_COLOR
local ColorPickerFrame = ColorPickerFrame

local _, addon = ...
local L = addon.L
local templates = {}
addon.optionTemplates = templates

-- Just do a "function" type check before calling, but I do need the error messages to popup!
local function _pcall(func, ...)
	if type(func) == "function" then
		return func(...)
	end
end

local pageCount = 0
function templates:CreateOptionFrame(name, title, version, desc, dualTalents)
	if type(name) ~= "string" then
		pageCount = pageCount + 1
		name = "CompactRaidOptionPage_UnnamedPage"..pageCount
	end

	local frame = UICreateInterfaceOptionPage(name, title, desc, nil, addon.optionFrame.pageContainer)
	frame.version = frame:CreateFontString(name.."Version", "ARTWORK", "GameFontNormalSmall")
	frame.version:SetPoint("LEFT", frame.title, "RIGHT", 8, 0)
	frame.version:SetText(version)

	if dualTalents then
		local dualTalentText = frame:CreateFontString(frame:GetName().."DualTalentText", "ARTWORK", "GameFontGreenSmall")
		frame.dualTalentText = dualTalentText
		dualTalentText:SetPoint("LEFT", frame.version, "RIGHT", 8, 0)
	end

	return frame
end

function templates:CreateScrollFrame(name, parent, scrollRespondingFrame, createChild)
	local frame = CreateFrame("ScrollFrame", name, parent, "UIPanelScrollFrameTemplate")
	frame.scrollBarHideable = 1

	local scrollBarBk = CreateFrame("Frame", nil, frame)
	scrollBarBk:SetBackdrop({ edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16, insets = {left = 5, right = 5, top = 5, bottom = 5 }})
	scrollBarBk:SetBackdropBorderColor(0.75, 0.75, 0.75, 0.75)
	scrollBarBk:SetPoint("TOPLEFT", _G[name.."ScrollBarScrollUpButton"], "TOPLEFT", -4, 4)
	scrollBarBk:SetPoint("BOTTOMRIGHT", _G[name.."ScrollBarScrollDownButton"], "BOTTOMRIGHT", 2, -4)
	scrollBarBk:SetParent(_G[name.."ScrollBar"])
	scrollBarBk:SetScale(0.6)

	local scrollChild
	if createChild then
		scrollChild = CreateFrame("Frame", name.."ScrollChild", frame)
		scrollChild:SetSize(100, 100)
		frame:SetScrollChild(scrollChild)
	end

	local respondFrame = scrollRespondingFrame or parent or frame
	respondFrame:EnableMouseWheel(true)
	respondFrame:SetScript("OnMouseWheel", function(self, delta) ScrollFrameTemplate_OnMouseWheel(frame, delta) end)

	frame:HookScript("OnScrollRangeChanged", function(self, xOffset, yOffset)
		if yOffset == 0 then
			_G[self:GetName().."ScrollBar"]:Hide()
		else
			_G[self:GetName().."ScrollBar"]:Show()
		end
	end)

	frame:HookScript("OnSizeChanged", function(self, x, y)
		if x then
			self:GetScrollChild():SetWidth(x)
		end
	end)

	return frame, scrollChild
end

local editToolltip = CreateFrame("Button", "CompactRaidOptionPageTooltip", UIParent)
templates.editToolltip = editToolltip

editToolltip:SetBackdrop({ edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16, insets = {left = 5, right = 5, top = 5, bottom = 5 }, })
--editToolltip:SetBackdropColor(TOOLTIP_DEFAULT_BACKGROUND_COLOR.r, TOOLTIP_DEFAULT_BACKGROUND_COLOR.g, TOOLTIP_DEFAULT_BACKGROUND_COLOR.b)
editToolltip:SetFrameStrata("TOOLTIP")
editToolltip:SetClampedToScreen(true)
editToolltip:SetWidth(240)

local tooltipBk = editToolltip:CreateTexture(nil, "BACKGROUND")
tooltipBk:SetPoint("TOPLEFT", 4, -4)
tooltipBk:SetPoint("BOTTOMRIGHT", -4, 4)
tooltipBk:SetTexture(0, 0, 0, 1)

editToolltip.text = editToolltip:CreateFontString(editToolltip:GetName().."Text", "ARTWORK", "GameFontHighlightSmallLeftTop")
editToolltip.text:SetPoint("TOPLEFT", 10, -10)
editToolltip.text:SetWidth(220)
editToolltip.text:SetTextColor(1, 0.5, 0)
editToolltip.text:SetNonSpaceWrap(true)
editToolltip:Hide()

function editToolltip:Display(editbox)
	self.text:SetText(editbox:IsMultiLine() and L["edit prompt multiline"] or L["edit prompt"])
	self:SetHeight(self.text:GetHeight() + 20)
	self:ClearAllPoints()
	self:SetPoint("TOPLEFT", editbox, "BOTTOMLEFT")
	self:Show()
end

function templates:SetEditBoxAutoPrompt(editbox)

	editbox:HookScript("OnEditFocusGained", function(self)
		editToolltip.owner = self
		editToolltip:Display(self)
	end)

	editbox:HookScript("OnEditFocusLost", function(self)
		if editToolltip.owner == self then
			editToolltip.owner = nil
			editToolltip:Hide()
		end
	end)
end

function templates:CreateSubEdit(parent, frame, text, key, disableInCombat, textColor)
	local edit = parent:CreateEditBox(text, true, disableInCombat, textColor)
	edit.key = key
	frame.editboxes[key] = edit
	edit:SetParent(frame)
	edit:SetWidth(50)
	edit:SetJustifyH("CENTER")
	edit.default = 0

	edit.text:ClearAllPoints()
	edit:SetPoint("LEFT", edit.text, "RIGHT", 2, 0)

	templates:SetEditBoxAutoPrompt(edit)

	edit.ValidateValue = function(self, value)
		if type(value) ~= "number" then
			return self.default
		elseif self.min and value < self.min then
			return self.min
		elseif self.max and value > self.max then
			return self.max
		end
	end

	edit.OnTextValidate = function(self, text)
		local err = self:ValidateValue(tonumber(text))
		if err then
			return 1, tostring(err)
		end
	end

	edit.OnTextCommit = function(self, text)
		self:GetParent():OnTextCommit(self, self:GetNumber())
	end

	edit.OnTextCancel = function(self)
		local value = self:GetParent():OnTextCancel(self)
		if value then
			return tostring(value)
		end
	end

	edit.Commit = function(self)
		local text = self:GetText()
		local invalid, newText = self:OnTextValidate(text)
		if invalid then
			text = newText
			self:SetText(text)
		end

		self:ClearFocus()
		return tonumber(text)
	end

	return edit
end

function templates:CreateScaleOffsetGroup(parent, disableInCombat, textColor)
	local frame = parent:CreateSubControl("Frame")
	frame:SetSize(26, 26)
	frame.editboxes = {}

	local scale = templates:CreateSubEdit(parent, frame, L["scale"], "scale", disableInCombat, textColor)
	scale.min, scale.max, scale.default = 10, 300, 100
	scale:SetNumeric(true)
	scale.text:ClearAllPoints()
	scale.text:SetPoint("LEFT", frame, "LEFT")
	scale:SetPoint("LEFT", scale.text, "RIGHT", 2, 0)

	local percLabel = scale:CreateFontString(nil, "ARTWORK", "GameFontNormalLeft")
	scale.percLabel = percLabel
	percLabel:SetText("%")
	percLabel:SetPoint("LEFT", scale, "RIGHT")

	local xoffset = templates:CreateSubEdit(parent, frame, L["x-offset"], "xoffset", disableInCombat, textColor)
	xoffset.text:SetPoint("LEFT", percLabel, "RIGHT", 8, 0)

	local yoffset =  templates:CreateSubEdit(parent, frame, L["y-offset"], "yoffset", disableInCombat, textColor)
	yoffset.text:SetPoint("LEFT", xoffset, "RIGHT", 8, 0)

	frame.OnTextCommit = function(self, editbox, value)
		if editbox.key == "scale" then
			_pcall(self.OnScaleApply, self, value)
		elseif editbox.key == "xoffset" or editbox.key == "yoffset" then
			if editbox.key == "xoffset" then
				_pcall(self.OnOffsetApply, self, value, self.editboxes.yoffset:Commit())
			elseif editbox.key == "yoffset" then
				_pcall(self.OnOffsetApply, self, self.editboxes.xoffset:Commit(), value)
			end
		end
	end

	frame.OnTextCancel = function(self, editbox)
		if editbox.key == "scale" then
			return _pcall(self.OnScaleCancel, self)
		elseif editbox.key == "xoffset" or editbox.key == "yoffset" then
			local x, y = _pcall(self.OnOffsetCancel, self)
			return editbox.key == "xoffset" and x or y
		end
	end

	frame.ClearFocus = function(self)
		local editbox
		for _, editbox in pairs(self.editboxes) do
			editbox:ClearFocus()
		end
	end

	frame.SetFocus = function(self, key)
		local editbox = self.editboxes[key]
		if editbox then
			editbox:SetFocus()
		end
	end

	frame.SetValue = function(self, key, value, value2)
		if key == "offset" then
			if type(value) ~= "number" then
				value = 0
			end
			self.editboxes.xoffset:SetNumber(value)

			if type(value2) ~= "number" then
				value2 = 0
			end
			self.editboxes.yoffset:SetNumber(value2)
			return value, value2
		else
			local edit = self.editboxes[key]
			if edit then
				local err = edit:ValidateValue(value)
				if err then
					value = err
				end
				edit:SetNumber(value)
				return value
			end
		end
	end

	return frame
end

-- Make the stupid ColorPickerFrame callback system a little bit object-oriented
local function SwatchFunc(restore)
	local swatch = templates.selectedColorSwatch
	if not swatch or not swatch[addon.guid] then
		return
	end

	local r, g, b
	if restore then
		r, g, b = restore.r, restore.g, restore.b
	else
		r, g, b = ColorPickerFrame:GetColorRGB()
	end

	swatch:SetColor(r, g, b)
	_pcall(swatch.OnColorChange, swatch, r, g, b)
	if restore then
		_pcall(swatch.OnColorCancel, swatch, r, g, b)
	end
end

function templates:CreateColorSwatch(name, parent)
	local colorSwatch = CreateFrame("Button", name, parent)
	colorSwatch[addon.guid] = 1
	colorSwatch:SetWidth(16)
	colorSwatch:SetHeight(16)
	colorSwatch.r, colorSwatch.g, colorSwatch.b = 0, 1, 0

	colorSwatch:SetBackdrop({ bgFile = "Interface\\Tooltips\\UI-Tooltip-Background", tile = true, tileSize = 8 })
	colorSwatch:SetBackdropColor(0, 0, 0, 1)

	colorSwatch.bg = colorSwatch:CreateTexture(nil, "BORDER")
	colorSwatch.bg:SetPoint("TOPLEFT", 1, -1)
	colorSwatch.bg:SetPoint("BOTTOMRIGHT", -1, 1)
	colorSwatch.bg:SetTexture(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)

	colorSwatch.swatch = colorSwatch:CreateTexture(nil, "ARTWORK")
	colorSwatch.swatch:SetPoint("TOPLEFT", 2, -2)
	colorSwatch.swatch:SetPoint("BOTTOMRIGHT", -2, 2)
	colorSwatch.swatch:SetTexture(0, 1, 0)

	colorSwatch._Enable = colorSwatch.Enable
	colorSwatch.Enable = function(self)
		self.swatch:SetTexture(self.r, self.g, self.b)
		self:_Enable()
		_pcall(self.OnEnable, self)
	end

	colorSwatch._Disable = colorSwatch.Disable
	colorSwatch.Disable = function(self)
		if templates.selectedColorSwatch == self and ColorPickerFrame.func == SwatchFunc then
			templates.selectedColorSwatch = nil
			ColorPickerFrame:Hide()
		end
		self.swatch:SetTexture(0.25, 0.25, 0.25)
		self:_Disable()
		_pcall(self.OnDisable, self)
	end

	colorSwatch.SetColor = function(self, r, g, b)
		self.r, self.g, self.b = r, g, b
		if self:IsEnabled() then
			self.swatch:SetTexture(r, g, b)
		end
	end

	colorSwatch.GetColor = function(self)
		return self.r, self.g, self.b
	end

	colorSwatch:SetScript("OnEnter", function(self)
		self.bg:SetTexture(NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b)
		GameTooltip:SetOwner(self, "ANCHOR_RIGHT")
		GameTooltip:ClearLines()
		GameTooltip:AddLine(L["click to change color"], nil, nil, nil, 1)
		GameTooltip:Show()
	end)

	colorSwatch:SetScript("OnLeave", function(self)
		GameTooltip:Hide()
		self.bg:SetTexture(HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b)
	end)

	colorSwatch:SetScript("OnClick", function(self)
		GameTooltip:Hide()
		ColorPickerFrame:Hide()
		templates.selectedColorSwatch = self
		ColorPickerFrame.func = SwatchFunc
		ColorPickerFrame:SetColorRGB(self.r, self.g, self.b)
		ColorPickerFrame.previousValues = { r = self.r, g = self.g, b = self.b }
		ColorPickerFrame.cancelFunc = SwatchFunc
		ColorPickerFrame.hasOpacity = nil
		ColorPickerFrame:Show()
	end)

	colorSwatch:SetScript("OnHide", function(self)
		if templates.selectedColorSwatch == self and ColorPickerFrame.func == SwatchFunc then
			templates.selectedColorSwatch = nil
			ColorPickerFrame:Hide()
		end
	end)

	return colorSwatch
end

function templates:CreateCustomColorGroup(parent, text)
	local group = parent:CreateSingleSelectionGroup(text, 1)
	group:AddButton(L["auto"], 1)
	local customeButton = group:AddButton(CUSTOM, 0)

	local colorSwatch = templates:CreateColorSwatch(customeButton:GetName().."ColorSwatch", customeButton)
	group.colorSwatch = colorSwatch
	colorSwatch:SetPoint("LEFT", customeButton.text, "RIGHT", 4, 0)

	colorSwatch.OnColorChange = function(self, r, g, b)
		_pcall(group.OnColorChange, group, r, g, b)
		_pcall(group.OnApplyOption, group)
	end

	colorSwatch:SetScript("OnShow", function(self)
		local r, g, b = _pcall(group.OnColorRequest, group)
		if not r then
			r, g, b = 0, 1, 0
		end
		self:SetColor(r, g, b)
	end)

	group.OnCheckInit = function(self, value)
		local checked = _pcall(self.OnOptionInit, self, value)
		if value == 1 then
			if checked then
				self.colorSwatch:Disable()
			else
				self.colorSwatch:Enable()
			end
		end
		return checked
	end

	group.OnSelectionChanged = function(self, value)
		if value == 1 then
			self.colorSwatch:Disable()
		else
			self.colorSwatch:Enable()
		end
		_pcall(self.OnOptionChange, self, value)
		_pcall(group.OnApplyOption, group)
	end

	return group
end

local function ClassOrderToolButton_OnClick(self)
	local parent = self:GetParent()
	local checkedIndex = parent.checked
	local checkedButton = parent.classButtons[checkedIndex]
	if not checkedButton or not checkedButton.class then
		return
	end

	local peerIndex = checkedIndex + self.operator
	local peerButton = parent.classButtons[peerIndex]
	if not peerButton then
		return
	end

	local class = checkedButton.class
	parent:SetButtonClass(checkedIndex, peerButton.class)
	parent:SetButtonClass(peerIndex, class)
	checkedButton:SetChecked(false)
	peerButton:SetChecked(true)
	parent.checked = peerIndex
	parent:SaveClassOrder()
end

function templates:CreateClassOrderFrame(name, parent, defaultOrder)
	local frame = CreateFrame("Frame", name, parent)
	frame:SetSize(248, 30)
	frame:SetBackdrop({ tile = true, tileSize = 16, edgeFile = "Interface\\Tooltips\\UI-Tooltip-Border", edgeSize = 16, insets = {left = 5, right = 5, top = 5, bottom = 5 }})
	frame.defaultOrder = defaultOrder
	frame.classButtons = {}

	local i, prev
	for i = 1, 10 do
		local button = CreateFrame("CheckButton", name.."Button"..i, frame)
		tinsert(frame.classButtons, button)
		button:SetID(i)
		button:SetSize(24, 24)
		button:SetPushedTexture("Interface\\Buttons\\UI-Quickslot-Depress")
		button:SetHighlightTexture("Interface\\Buttons\\ButtonHilight-Square", "ADD")
		button:SetCheckedTexture("Interface\\Buttons\\CheckButtonHilight", "ADD")

		local icon = button:CreateTexture(nil, "ARTWORK")
		button.icon = icon
		icon:SetAllPoints(button)
		icon:SetTexture("Interface\\WorldStateFrame\\Icons-Classes")

		button:SetScript("OnClick", function(self)
			local parent = self:GetParent()
			if self:GetChecked() then
				if parent.checked and parent.checked ~= self:GetID() then
					local peer = parent.classButtons[parent.checked]
					if peer then
						peer:SetChecked(false)
					end
				end
				parent.checked = self:GetID()
			else
				parent.checked = nil
			end
			parent:UpdateUtilButtonStatus()
		end)

		if prev then
			button:SetPoint("LEFT", prev, "RIGHT")
		else
			button:SetPoint("LEFT", 4, 0)
		end
		prev = button
	end

	frame.UpdateUtilButtonStatus = function(self)
		local combat = InCombatLockdown()
		local checkedButton = self.classButtons[self.checked]
		if combat or not checkedButton or not checkedButton.class then
			self.forwardButton:Disable()
			self.backwardButton:Disable()
		else
			if self.checked == 1 then
				self.forwardButton:Disable()
			else
				self.forwardButton:Enable()
			end

			if self.checked == 10 then
				self.backwardButton:Disable()
			else
				self.backwardButton:Enable()
			end
		end

		if combat or self.classOrder == self.defaultOrder then
			self.classResetButton:Disable()
		else
			self.classResetButton:Enable()
		end
	end

	frame.SetButtonClass = function(self, index, class)
		local button = self.classButtons[index]
		if not button then
			return
		end

		local coords = CLASS_BUTTONS[class]
		if coords then
			button.class = class
			button.icon:SetTexCoord(coords[1], coords[2], coords[3], coords[4])
			button.icon:Show()
		else
			button.class = nil
			button.icon:Hide()
		end
	end

	frame.LoadClassOrder = function(self, classOrder)
		if type(classOrder) ~= "string" then
			return
		end

		self.classOrder = classOrder
		local temp = { strsplit(",", classOrder) }
		local i
		for i = 1, 10 do
			self:SetButtonClass(i, temp[i])
		end
	end

	frame.SaveClassOrder = function(self)
		local result, i
		for i = 1, 10 do
			local class = self.classButtons[i].class
			if class then
				if result then
					result = result..","..class
				else
					result = class
				end
			end
		end
		self.classOrder = result
		self:UpdateUtilButtonStatus()
		if type(self.OnClassOrderChanged) == "function" then
			self:OnClassOrderChanged(result)
		end
	end

	frame:SetScript("OnShow", function(self)
		self:SetScript("OnShow", nil)
		if type(self.OnClassOrderInit) == "function" then
			local classOrder = self:OnClassOrderInit()
			self:LoadClassOrder(classOrder)
		end
		self:UpdateUtilButtonStatus()
	end)

	local forwardButton = CreateFrame("Button", name.."Forward", frame, "UIPanelButtonTemplate")
	forwardButton.operator = -1
	forwardButton:SetSize(36, 24)
	forwardButton:SetPoint("LEFT", frame, "RIGHT", 6, 0)
	forwardButton:SetText("<")
	forwardButton:SetScript("OnClick", ClassOrderToolButton_OnClick)

	local backwardButton = CreateFrame("Button", name.."Backward", frame, "UIPanelButtonTemplate")
	backwardButton.operator = 1
	backwardButton:SetSize(36, 24)
	backwardButton:SetPoint("LEFT", forwardButton, "RIGHT")
	backwardButton:SetText(">")
	backwardButton:SetScript("OnClick", ClassOrderToolButton_OnClick)

	local classResetButton = CreateFrame("Button", name.."Reset", frame, "UIPanelButtonTemplate")
	classResetButton:SetSize(80, 24)
	classResetButton:SetPoint("LEFT", backwardButton, "RIGHT", 6, 0)
	classResetButton:SetText(RESET)

	classResetButton:SetScript("OnClick", function(self)
		local parent = self:GetParent()
		parent:LoadClassOrder(parent.defaultOrder)
		local i
		for i = 1, 10 do
			parent.classButtons[i]:SetChecked(false)
		end
		parent.checked = nil
		parent.forwardButton:Disable()
		parent.backwardButton:Disable()
		self:Disable()
		parent:SaveClassOrder()
	end)

	frame.forwardButton, frame.backwardButton, frame.classResetButton = forwardButton, backwardButton, classResetButton

	frame:RegisterEvent("PLAYER_REGEN_ENABLED")
	frame:RegisterEvent("PLAYER_REGEN_DISABLED")
	frame:SetScript("OnEvent", frame.UpdateUtilButtonStatus)

	return frame
end