------------------------------------------------------------
-- RaidFrames.lua
--
-- Abin
-- 2012/1/03
------------------------------------------------------------

local _, addon = ...
local L = addon.L

local DEFAULT_CLASS_ORDER = "WARRIOR,PALADIN,DEATHKNIGHT,DRUID,PRIEST,SHAMAN,HUNTER,ROGUE,MAGE,WARLOCK"

function addon:GetDefaultClassOrder()
	return DEFAULT_CLASS_ORDER
end

local groupHeaders = {}

local classFrame = CreateFrame("Frame", "CompactRaidGroupHeaderParentClass", addon:GetMainFrame(), "SecureFrameTemplate")
classFrame:SetSize(16, 16)
classFrame:SetPoint("TOPLEFT")

local groupFrame = CreateFrame("Frame", "CompactRaidGroupHeaderParentGroup", addon:GetMainFrame(), "SecureFrameTemplate")
groupFrame:SetSize(16, 16)
groupFrame:SetPoint("TOPLEFT")

local i, j
for i = 0, 8 do
	local header = CreateFrame("Frame", "CompactRaidGroupHeader"..i, i == 0 and classFrame or groupFrame, "SecureRaidGroupHeaderTemplate")
	groupHeaders[i] = header
	header:SetPoint("TOPLEFT", i < 2 and 0 or (i - 1) * 64 + (i - 1), 0)
	header:SetAttribute("template", addon:GetTemplates())
	header:SetAttribute("point", "TOP")
	header:SetAttribute("xOffset", 0)
	header:SetAttribute("yOffset", -1)
	header:SetAttribute("unitsPerColumn", 5)
	header:SetAttribute("columnAnchorPoint", "LEFT")
	header:SetAttribute("columnSpacing", 1)

	local count
	if i > 0 then
		count = 5
		header:SetAttribute("groupFilter", i)
		header:SetAttribute("maxColumns", 1)
		header:SetAttribute("startingIndex", -4)
	else
		count = 40
		header:SetAttribute("groupBy", "CLASS")
		header:SetAttribute("groupingOrder", addon:GetDefaultClassOrder())
		header:SetAttribute("maxColumns", 8)
		header:SetAttribute("startingIndex", -39)
	end

	header:Show()
	header:SetAttribute("startingIndex", 1) -- Restore normal index

	for j = 1, #header do
		local button = header[j]
		addon.__InitUnitFrame(button)
		if i > 1 or j > 1 then
			button.affectContainer = 1
		end
	end
end
addon.__InitUnitFrame = nil

function addon:UpdateRaidGroupFilter()
	if addon.db.classfilter == 1 then
		classFrame:Show()
		groupFrame:Hide()
	else
		classFrame:Hide()
		groupFrame:Show()
	end

	addon:UpdateContainerSize()
end

function addon:UpdateRaidGroupClassOrder()
	groupHeaders[0]:SetAttribute("groupingOrder", addon.db.classorder)
end

function addon:UpdateRaidGroupHeaderSize()
	local width, height, spacing = addon.db.width, addon.db.height, addon.db.spacing
	local i
	for i = 2, 8 do
		local header = groupHeaders[i]
		header:ClearAllPoints()
		if addon.db.raidhoriz then
			header:SetPoint("TOPLEFT", 0, 0 - (i - 1) * (height + spacing))
		else
			header:SetPoint("TOPLEFT", (i - 1) * (width + spacing), 0)
		end
	end
	addon:UpdateContainerSize()
end

function addon:UpdateRaidGroupOrientation()
	local spacing = addon.db.spacing
	local horiz = addon.db.raidhoriz
	local i, j
	for i = 0, 8 do
		local header = groupHeaders[i]
		if horiz then
			header:SetAttribute("columnAnchorPoint", "TOP")
			header:SetAttribute("point", "LEFT")
			header:SetAttribute("xOffset", spacing)
			header:SetAttribute("yOffset", 0)
		else
			header:SetAttribute("columnAnchorPoint", "LEFT")
			header:SetAttribute("point", "TOP")
			header:SetAttribute("xOffset", 0)
			header:SetAttribute("yOffset", -spacing)
		end

		-- Fix a stupid bug for Blizzard, they forgot to call "ClearAllPoints" before calling "SetPoint", eh?
		for j = 1, #header do
			header[j]:ClearAllPoints()
		end
		header:SetAttribute("unitsPerColumn", 5)
	end

	addon:UpdateRaidGroupHeaderSize()
	addon:UpdateContainerSize()
end

function addon:UpdateRaidGroupSpacing()
	addon:UpdateRaidGroupOrientation()
end

function addon:GetRaidGroup(index)
	return groupHeaders[index]
end

function addon:GetRaidGroupVisibleSize()
	if addon.db.classfilter == 1 then
		local header = groupHeaders[0]
		return header:GetWidth(), header:GetHeight()
	end

	local x, y, spacing = addon.db.width, addon.db.height, addon.db.spacing
	local width, height = 0, 0
	local maxGroup, maxUnit = 0, 0
	local i, j
	for i = 1, 8 do
		local group = groupHeaders[i]
		for j = 1, 5 do
			local button = group[j]
			if button:IsVisible() then
				if maxGroup < i then
					maxGroup = i
				end

				if maxUnit < j then
					maxUnit = j
				end
			else
				break
			end
		end
	end

	if addon.db.raidhoriz then
		width = maxUnit * x + (maxUnit - 1) * spacing
		height = maxGroup * y + (maxGroup - 1) * spacing
	else
		width = maxGroup * x + (maxGroup - 1) * spacing
		height = maxUnit * y + (maxUnit - 1) * spacing
	end
	return width, height
end